@@ -19,9 +19,11 @@ to get the difference.
 import asyncio
 import datetime
 import time
+import logging
 from enum import Enum
 from .session import SessionState, ChannelState
 from ..tl import types as tl, functions as fn
+from ..helpers import get_running_loop
 
 
 # Telegram sends `seq` equal to `0` when "it doesn't matter", so we use that value too.
@@ -43,18 +45,34 @@ POSSIBLE_GAP_TIMEOUT = 0.5
 # Documentation recommends 15 minutes without updates (https://core.telegram.org/api/updates).
 NO_UPDATES_TIMEOUT = 15 * 60
 
+# object() but with a tag to make it easier to debug
+class Sentinel:
+    __slots__ = ('tag',)
+
+    def __init__(self, tag=None):
+        self.tag = tag or '_'
+
+    def __repr__(self):
+        return self.tag
+
 # Entry "enum".
 # Account-wide `pts` includes private conversations (one-to-one) and small group chats.
-ENTRY_ACCOUNT = object()
+ENTRY_ACCOUNT = Sentinel('ACCOUNT')
 # Account-wide `qts` includes only "secret" one-to-one chats.
-ENTRY_SECRET = object()
+ENTRY_SECRET = Sentinel('SECRET')
 # Integers will be Channel-specific `pts`, and includes "megagroup", "broadcast" and "supergroup" channels.
 
-_sentinel = object()
+# Python's logging doesn't define a TRACE level. Pick halfway between DEBUG and NOTSET.
+# We don't define a name for this as libraries shouldn't do that though.
+LOG_LEVEL_TRACE = (logging.DEBUG - logging.NOTSET) // 2
+
+_sentinel = Sentinel()
 
 def next_updates_deadline():
-    return asyncio.get_running_loop().time() + NO_UPDATES_TIMEOUT
+    return get_running_loop().time() + NO_UPDATES_TIMEOUT
 
+def epoch():
+    return datetime.datetime(*time.gmtime(0)[:6]).replace(tzinfo=datetime.timezone.utc)
 
 class GapError(ValueError):
     def __repr__(self):
@@ -93,19 +111,12 @@ class PtsInfo:
 
         qts = getattr(update, 'qts', None)
         if qts:
-            pts_count = 1 if isinstance(update, tl.UpdateNewEncryptedMessage) else 0
-            return cls(pts=qts, pts_count=pts_count, entry=ENTRY_SECRET)
+            return cls(pts=qts, pts_count=1, entry=ENTRY_SECRET)
 
         return None
 
     def __repr__(self):
-        if self.entry is ENTRY_ACCOUNT:
-            entry = 'ENTRY_ACCOUNT'
-        elif self.entry is ENTRY_SECRET:
-            entry = 'ENTRY_SECRET'
-        else:
-            entry = self.entry
-        return f'PtsInfo(pts={self.pts}, pts_count={self.pts_count}, entry={entry})'
+        return f'PtsInfo(pts={self.pts}, pts_count={self.pts_count}, entry={self.entry})'
 
 
 # The state of a particular entry in the message box.
@@ -154,15 +165,16 @@ class PossibleGap:
 #
 # See https://core.telegram.org/api/updates#message-related-event-sequences.
 class MessageBox:
-    __slots__ = ('map', 'date', 'seq', 'next_deadline', 'possible_gaps', 'getting_diff_for', 'reset_deadlines_for')
+    __slots__ = ('_log', 'map', 'date', 'seq', 'next_deadline', 'possible_gaps', 'getting_diff_for')
 
     def __init__(
         self,
+        log,
         # Map each entry to their current state.
         map: dict = _sentinel,  # entry -> state
 
         # Additional fields beyond PTS needed by `ENTRY_ACCOUNT`.
-        date: datetime.datetime = datetime.datetime(*time.gmtime(0)[:6]).replace(tzinfo=datetime.timezone.utc),
+        date: datetime.datetime = epoch() + datetime.timedelta(seconds=1),
         seq: int = NO_SEQ,
 
         # Holds the entry with the closest deadline (optimization to avoid recalculating the minimum deadline).
@@ -179,18 +191,26 @@ class MessageBox:
 
         # For which entries are we currently getting difference.
         getting_diff_for: set = _sentinel,  # entry
-
-        # Temporarily stores which entries should have their update deadline reset.
-        # Stored in the message box in order to reuse the allocation.
-        reset_deadlines_for: set = _sentinel  # entry
     ):
+        self._log = log
         self.map = {} if map is _sentinel else map
         self.date = date
         self.seq = seq
         self.next_deadline = next_deadline
         self.possible_gaps = {} if possible_gaps is _sentinel else possible_gaps
         self.getting_diff_for = set() if getting_diff_for is _sentinel else getting_diff_for
-        self.reset_deadlines_for = set() if reset_deadlines_for is _sentinel else reset_deadlines_for
+
+        if __debug__:
+            self._trace('MessageBox initialized')
+
+    def _trace(self, msg, *args, **kwargs):
+        # Calls to trace can't really be removed beforehand without some dark magic.
+        # So every call to trace is prefixed with `if __debug__`` instead, to remove
+        # it when using `python -O`. Probably unnecessary, but it's nice to avoid
+        # paying the cost for something that is not used.
+        self._log.log(LOG_LEVEL_TRACE, 'Current MessageBox state: seq = %r, date = %s, map = %r',
+                      self.seq, self.date.isoformat(), self.map)
+        self._log.log(LOG_LEVEL_TRACE, msg, *args, **kwargs)
 
     # region Creation, querying, and setting base state.
 
@@ -198,6 +218,9 @@ class MessageBox:
         """
         Create a [`MessageBox`] from a previously known update state.
         """
+        if __debug__:
+            self._trace('Loading MessageBox with session_state = %r, channel_states = %r', session_state, channel_states)
+
         deadline = next_updates_deadline()
 
         self.map.clear()
@@ -207,7 +230,7 @@ class MessageBox:
             self.map[ENTRY_SECRET] = State(pts=session_state.qts, deadline=deadline)
         self.map.update((s.channel_id, State(pts=s.pts, deadline=deadline)) for s in channel_states)
 
-        self.date = datetime.datetime.fromtimestamp(session_state.date).replace(tzinfo=datetime.timezone.utc)
+        self.date = datetime.datetime.fromtimestamp(session_state.date, tz=datetime.timezone.utc)
         self.seq = session_state.seq
         self.next_deadline = ENTRY_ACCOUNT
 
@@ -237,7 +260,7 @@ class MessageBox:
         If a deadline expired, the corresponding entries will be marked as needing to get its difference.
         While there are entries pending of getting their difference, this method returns the current instant.
         """
-        now = asyncio.get_running_loop().time()
+        now = get_running_loop().time()
 
         if self.getting_diff_for:
             return now
@@ -255,8 +278,11 @@ class MessageBox:
         # timeout for updates several times (it also makes sense to get difference if now is the deadline).
         if now >= deadline:
             # Check all expired entries and add them to the list that needs getting difference.
-            self.getting_diff_for.update(entry for entry, gap in self.possible_gaps.items() if now > gap.deadline)
-            self.getting_diff_for.update(entry for entry, state in self.map.items() if now > state.deadline)
+            self.getting_diff_for.update(entry for entry, gap in self.possible_gaps.items() if now >= gap.deadline)
+            self.getting_diff_for.update(entry for entry, state in self.map.items() if now >= state.deadline)
+
+            if __debug__:
+                self._trace('Deadlines met, now getting diff for %r', self.getting_diff_for)
 
             # When extending `getting_diff_for`, it's important to have the moral equivalent of
             # `begin_get_diff` (that is, clear possible gaps if we're now getting difference).
@@ -265,44 +291,38 @@ class MessageBox:
 
         return deadline
 
-    # Reset the deadline for the periods without updates for a given entry.
+    # Reset the deadline for the periods without updates for the given entries.
     #
     # It also updates the next deadline time to reflect the new closest deadline.
-    def reset_deadline(self, entry, deadline):
-        if entry not in self.map:
-            raise RuntimeError('Called reset_deadline on an entry for which we do not have state')
-        self.map[entry].deadline = deadline
+    def reset_deadlines(self, entries, deadline):
+        if not entries:
+            return
+        for entry in entries:
+            if entry not in self.map:
+                raise RuntimeError('Called reset_deadline on an entry for which we do not have state')
+            self.map[entry].deadline = deadline
 
-        if self.next_deadline == entry:
+        if self.next_deadline in entries:
             # If the updated deadline was the closest one, recalculate the new minimum.
             self.next_deadline = min(self.map.items(), key=lambda entry_state: entry_state[1].deadline)[0]
         elif self.next_deadline in self.map and deadline < self.map[self.next_deadline].deadline:
             # If the updated deadline is smaller than the next deadline, change the next deadline to be the new one.
+            # Any entry will do, so the one from the last iteration is fine.
             self.next_deadline = entry
         # else an unrelated deadline was updated, so the closest one remains unchanged.
 
     # Convenience to reset a channel's deadline, with optional timeout.
     def reset_channel_deadline(self, channel_id, timeout):
-        self.reset_deadline(channel_id, asyncio.get_running_loop().time() + (timeout or NO_UPDATES_TIMEOUT))
-
-    # Reset all the deadlines in `reset_deadlines_for` and then empty the set.
-    def apply_deadlines_reset(self):
-        next_deadline = next_updates_deadline()
-
-        reset_deadlines_for = self.reset_deadlines_for
-        self.reset_deadlines_for = set()  # "move" the set to avoid self.reset_deadline() from touching it during iter
-
-        for entry in reset_deadlines_for:
-            self.reset_deadline(entry, next_deadline)
-
-        reset_deadlines_for.clear()  # reuse allocation, the other empty set was a temporary dummy value
-        self.reset_deadlines_for = reset_deadlines_for
+        self.reset_deadlines({channel_id}, get_running_loop().time() + (timeout or NO_UPDATES_TIMEOUT))
 
     # Sets the update state.
     #
     # Should be called right after login if [`MessageBox::new`] was used, otherwise undesirable
     # updates will be fetched.
     def set_state(self, state, reset=True):
+        if __debug__:
+            self._trace('Setting state %s', state)
+
         deadline = next_updates_deadline()
 
         if state.pts != NO_SEQ or not reset:
@@ -329,6 +349,9 @@ class MessageBox:
     #
     # The update state will only be updated if no entry was known previously.
     def try_set_channel_state(self, id, pts):
+        if __debug__:
+            self._trace('Trying to set channel state for %r: %r', id, pts)
+
         if id not in self.map:
             self.map[id] = State(pts=pts, deadline=next_updates_deadline())
 
@@ -336,15 +359,18 @@ class MessageBox:
     # Fails if the entry does not have a previously-known state that can be used to get its difference.
     #
     # Clears any previous gaps.
-    def try_begin_get_diff(self, entry):
+    def try_begin_get_diff(self, entry, reason):
         if entry not in self.map:
             # Won't actually be able to get difference for this entry if we don't have a pts to start off from.
             if entry in self.possible_gaps:
                 raise RuntimeError('Should not have a possible_gap for an entry not in the state map')
 
-            # TODO it would be useful to log when this happens
+            if __debug__:
+                self._trace('Should get difference for %r because %s but cannot due to missing hash', entry, reason)
             return
 
+        if __debug__:
+            self._trace('Marking %r as needing difference because %s', entry, reason)
         self.getting_diff_for.add(entry)
         self.possible_gaps.pop(entry, None)
 
@@ -357,7 +383,7 @@ class MessageBox:
         except KeyError:
             raise RuntimeError('Called end_get_diff on an entry which was not getting diff for')
 
-        self.reset_deadline(entry, next_updates_deadline())
+        self.reset_deadlines({entry}, next_updates_deadline())
         assert entry not in self.possible_gaps, "gaps shouldn't be created while getting difference"
 
     # endregion Creation, querying, and setting base state.
@@ -382,11 +408,6 @@ class MessageBox:
         chat_hashes,
         result,  # out list of updates; returns list of user, chat, or raise if gap
     ):
-        date = getattr(updates, 'date', None)
-        if date is None:
-            # updatesTooLong is the only one with no date (we treat it as a gap)
-            self.try_begin_get_diff(ENTRY_ACCOUNT)
-            raise GapError
 
         # v1 has never sent updates produced by the client itself to the handlers.
         # However proper update handling requires those to be processed.
@@ -395,11 +416,25 @@ class MessageBox:
         real_result = result
         result = []
 
-        seq = getattr(updates, 'seq', None) or NO_SEQ
-        seq_start = getattr(updates, 'seq_start', None) or seq
+        date = getattr(updates, 'date', None)
+        seq = getattr(updates, 'seq', None)
+        seq_start = getattr(updates, 'seq_start', None)
         users = getattr(updates, 'users', None) or []
         chats = getattr(updates, 'chats', None) or []
 
+        if __debug__:
+            self._trace('Processing updates with seq = %r, seq_start = %r, date = %s: %s',
+                        seq, seq_start, date.isoformat() if date else None, updates)
+
+        if date is None:
+            # updatesTooLong is the only one with no date (we treat it as a gap)
+            self.try_begin_get_diff(ENTRY_ACCOUNT, 'received updatesTooLong')
+            raise GapError
+        if seq is None:
+            seq = NO_SEQ
+        if seq_start is None:
+            seq_start = seq
+
         # updateShort is the only update which cannot be dispatched directly but doesn't have 'updates' field
         updates = getattr(updates, 'updates', None) or [updates.update if isinstance(updates, tl.UpdateShort) else updates]
 
@@ -411,26 +446,50 @@ class MessageBox:
         if seq_start != NO_SEQ:
             if self.seq + 1 > seq_start:
                 # Skipping updates that were already handled
+                if __debug__:
+                    self._trace('Skipping updates as they should have already been handled')
                 return (users, chats)
             elif self.seq + 1 < seq_start:
                 # Gap detected
-                self.try_begin_get_diff(ENTRY_ACCOUNT)
+                self.try_begin_get_diff(ENTRY_ACCOUNT, 'detected gap')
                 raise GapError
             # else apply
 
-            self.date = date
-            if seq != NO_SEQ:
-                self.seq = seq
-
-        result.extend(filter(None, (self.apply_pts_info(u, reset_deadline=True) for u in updates)))
-
-        self.apply_deadlines_reset()
-
         def _sort_gaps(update):
             pts = PtsInfo.from_update(update)
             return pts.pts - pts.pts_count if pts else 0
 
+        reset_deadlines = set()  # temporary buffer
+        any_pts_applied = [False]  # using a list to pass "by reference"
+
+        result.extend(filter(None, (
+            self.apply_pts_info(u, reset_deadlines=reset_deadlines, any_pts_applied=any_pts_applied)
+            # Telegram can send updates out of order (e.g. ReadChannelInbox first
+            # and then NewChannelMessage, both with the same pts, but the count is
+            # 0 and 1 respectively), so we sort them first.
+            for u in sorted(updates, key=_sort_gaps))))
+
+        # > If the updates were applied, local *Updates* state must be updated
+        # > with `seq` (unless it's 0) and `date` from the constructor.
+        #
+        # By "were applied", we assume it means "some other pts was applied".
+        # Updates which can be applied in any order, such as `UpdateChat`,
+        # should not cause `seq` to be updated (or upcoming updates such as
+        # `UpdateChatParticipant` could be missed).
+        if any_pts_applied[0]:
+            if __debug__:
+                self._trace('Updating seq as local pts was updated too')
+            if date != epoch():
+                self.date = date
+            if seq != NO_SEQ:
+                self.seq = seq
+
+        self.reset_deadlines(reset_deadlines, next_updates_deadline())
+
         if self.possible_gaps:
+            if __debug__:
+                self._trace('Trying to re-apply %r possible gaps', len(self.possible_gaps))
+
             # For each update in possible gaps, see if the gap has been resolved already.
             for key in list(self.possible_gaps.keys()):
                 self.possible_gaps[key].updates.sort(key=_sort_gaps)
@@ -440,9 +499,11 @@ class MessageBox:
 
                     # If this fails to apply, it will get re-inserted at the end.
                     # All should fail, so the order will be preserved (it would've cycled once).
-                    update = self.apply_pts_info(update, reset_deadline=False)
+                    update = self.apply_pts_info(update, reset_deadlines=None)
                     if update:
                         result.append(update)
+                        if __debug__:
+                            self._trace('Resolved gap with %r: %s', PtsInfo.from_update(update), update)
 
             # Clear now-empty gaps.
             self.possible_gaps = {entry: gap for entry, gap in self.possible_gaps.items() if gap.updates}
@@ -460,16 +521,19 @@ class MessageBox:
         self,
         update,
         *,
-        reset_deadline,
+        reset_deadlines,
+        any_pts_applied=[True],  # mutable default is fine as it's write-only
     ):
         # This update means we need to call getChannelDifference to get the updates from the channel
         if isinstance(update, tl.UpdateChannelTooLong):
-            self.try_begin_get_diff(update.channel_id)
+            self.try_begin_get_diff(update.channel_id, 'received updateChannelTooLong')
             return None
 
         pts = PtsInfo.from_update(update)
         if not pts:
             # No pts means that the update can be applied in any order.
+            if __debug__:
+                self._trace('No pts in update, so it can be applied in any order: %s', update)
             return update
 
         # As soon as we receive an update of any form related to messages (has `PtsInfo`),
@@ -478,25 +542,31 @@ class MessageBox:
         # Build the `HashSet` to avoid calling `reset_deadline` more than once for the same entry.
         #
         # By the time this method returns, self.map will have an entry for which we can reset its deadline.
-        if reset_deadline:
-            self.reset_deadlines_for.add(pts.entry)
+        if reset_deadlines:
+            reset_deadlines.add(pts.entry)
 
         if pts.entry in self.getting_diff_for:
             # Note: early returning here also prevents gap from being inserted (which they should
             # not be while getting difference).
+            if __debug__:
+                self._trace('Skipping update with %r as its difference is being fetched', pts)
             return None
 
         if pts.entry in self.map:
             local_pts = self.map[pts.entry].pts
             if local_pts + pts.pts_count > pts.pts:
                 # Ignore
+                if __debug__:
+                    self._trace('Skipping update since local pts %r > %r: %s', local_pts, pts, update)
                 return None
             elif local_pts + pts.pts_count < pts.pts:
                 # Possible gap
                 # TODO store chats too?
+                if __debug__:
+                    self._trace('Possible gap since local pts %r < %r: %s', local_pts, pts, update)
                 if pts.entry not in self.possible_gaps:
                     self.possible_gaps[pts.entry] = PossibleGap(
-                        deadline=asyncio.get_running_loop().time() + POSSIBLE_GAP_TIMEOUT,
+                        deadline=get_running_loop().time() + POSSIBLE_GAP_TIMEOUT,
                         updates=[]
                     )
 
@@ -504,24 +574,37 @@ class MessageBox:
                 return None
             else:
                 # Apply
-                pass
-        else:
-            # No previous `pts` known, and because this update has to be "right" (it's the first one) our
-            # `local_pts` must be the one before the server pts.
-            local_pts = pts.pts - pts.pts_count
+                any_pts_applied[0] = True
+                if __debug__:
+                    self._trace('Applying update pts since local pts %r = %r: %s', local_pts, pts, update)
 
         # In a channel, we may immediately receive:
-        # * ReadChannelInbox (pts = X)
+        # * ReadChannelInbox (pts = X, pts_count = 0)
         # * NewChannelMessage (pts = X, pts_count = 1)
         #
-        # Notice how both `pts` are the same. The first one however would've triggered a gap
-        # because `local_pts` + `pts_count` of 0 would be less than `remote_pts`. So there is
-        # no risk by setting the `local_pts` to match the `remote_pts` here of missing the new
-        # message.
+        # Notice how both `pts` are the same. If they were to be applied out of order, the first
+        # one however would've triggered a gap because `local_pts` + `pts_count` of 0 would be
+        # less than `remote_pts`. So there is no risk by setting the `local_pts` to match the
+        # `remote_pts` here of missing the new message.
+        #
+        # The message would however be lost if we initialized the pts with the first one, since
+        # the second one would appear "already handled". To prevent this we set the pts to be
+        # one less when the count is 0 (which might be wrong and trigger a gap later on, but is
+        # unlikely). This will prevent us from losing updates in the unlikely scenario where these
+        # two updates arrive in different packets (and therefore couldn't be sorted beforehand).
         if pts.entry in self.map:
             self.map[pts.entry].pts = pts.pts
         else:
-            self.map[pts.entry] = State(pts=pts.pts, deadline=next_updates_deadline())
+            # When a chat is migrated to a megagroup, the first update can be a `ReadChannelInbox`
+            # with `pts = 1, pts_count = 0` followed by a `NewChannelMessage` with `pts = 2, pts_count=1`.
+            # Note how the `pts` for the message is 2 and not 1 unlike the case described before!
+            # This is likely because the `pts` cannot be 0 (or it would fail with PERSISTENT_TIMESTAMP_EMPTY),
+            # which forces the first update to be 1. But if we got difference with 1 and the second update
+            # also used 1, we would miss it, so Telegram probably uses 2 to work around that.
+            self.map[pts.entry] = State(
+                pts=(pts.pts - (0 if pts.pts_count else 1)) or 1,
+                deadline=next_updates_deadline()
+            )
 
         return update
 
@@ -536,12 +619,15 @@ class MessageBox:
                 if entry not in self.map:
                     raise RuntimeError('Should not try to get difference for an entry without known state')
 
-                return fn.updates.GetDifferenceRequest(
+                gd = fn.updates.GetDifferenceRequest(
                     pts=self.map[ENTRY_ACCOUNT].pts,
                     pts_total_limit=None,
                     date=self.date,
                     qts=self.map[ENTRY_SECRET].pts if ENTRY_SECRET in self.map else NO_SEQ,
                 )
+                if __debug__:
+                    self._trace('Requesting account difference %s', gd)
+                return gd
 
         return None
 
@@ -551,6 +637,9 @@ class MessageBox:
         diff,
         chat_hashes,
     ):
+        if __debug__:
+            self._trace('Applying account difference %s', diff)
+
         finish = None
         result = None
 
@@ -577,7 +666,7 @@ class MessageBox:
             secret = ENTRY_SECRET in self.getting_diff_for
 
             if not account and not secret:
-                raise RuntimeWarning('Should not be applying the difference when neither account or secret was diff was active')
+                raise RuntimeError('Should not be applying the difference when neither account or secret was diff was active')
 
             # Both may be active if both expired at the same time.
             if account:
@@ -602,7 +691,7 @@ class MessageBox:
             updates=diff.other_updates,
             users=diff.users,
             chats=diff.chats,
-            date=1,  # anything not-None
+            date=epoch(),
             seq=NO_SEQ,  # this way date is not used
         ), chat_hashes, updates)
 
@@ -619,11 +708,14 @@ class MessageBox:
         return updates, diff.users, diff.chats
 
     def end_difference(self):
+        if __debug__:
+            self._trace('Ending account difference')
+
         account = ENTRY_ACCOUNT in self.getting_diff_for
         secret = ENTRY_SECRET in self.getting_diff_for
 
         if not account and not secret:
-            raise RuntimeWarning('Should not be ending get difference when neither account or secret was diff was active')
+            raise RuntimeError('Should not be ending get difference when neither account or secret was diff was active')
 
         # Both may be active if both expired at the same time.
         if account:
@@ -658,13 +750,16 @@ class MessageBox:
         if not state:
             raise RuntimeError('Should not try to get difference for an entry without known state')
 
-        return fn.updates.GetChannelDifferenceRequest(
+        gd = fn.updates.GetChannelDifferenceRequest(
             force=False,
             channel=tl.InputChannel(packed.id, packed.hash),
             filter=tl.ChannelMessagesFilterEmpty(),
             pts=state.pts,
             limit=BOT_CHANNEL_DIFF_LIMIT if chat_hashes.self_bot else USER_CHANNEL_DIFF_LIMIT
         )
+        if __debug__:
+            self._trace('Requesting channel difference %s', gd)
+        return gd
 
     # Similar to [`MessageBox::process_updates`], but using the result from getting difference.
     def apply_channel_difference(
@@ -674,6 +769,9 @@ class MessageBox:
         chat_hashes,
     ):
         entry = request.channel.channel_id
+        if __debug__:
+            self._trace('Applying channel difference for %r: %s', entry, diff)
+
         self.possible_gaps.pop(entry, None)
 
         if isinstance(diff, tl.updates.ChannelDifferenceEmpty):
@@ -702,7 +800,7 @@ class MessageBox:
                 updates=diff.other_updates,
                 users=diff.users,
                 chats=diff.chats,
-                date=1,  # anything not-None
+                date=epoch(),
                 seq=NO_SEQ,  # this way date is not used
             ), chat_hashes, updates)
 
@@ -717,6 +815,8 @@ class MessageBox:
 
     def end_channel_difference(self, request, reason: PrematureEndReason, chat_hashes):
         entry = request.channel.channel_id
+        if __debug__:
+            self._trace('Ending channel difference for %r because %s', entry, reason)
 
         if reason == PrematureEndReason.TEMPORARY_SERVER_ISSUES:
             # Temporary issues. End getting difference without updating the pts so we can retry later.
